#!/usr/bin/python3
from requests.auth import HTTPBasicAuth
from getpass import getpass
from urllib.parse import urlparse
import argparse, logging
import WPImport, WPExport


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("--debug", help="Verbosity",  action="store_true")
    parser.add_argument("--logfile", help="Log file",  default="")
    parser.add_argument("--quiet", help="No console output", action="store_true")
    parser.add_argument("--parser", help="Parser content", default="html.parser")

    subparsers = parser.add_subparsers(dest="command")

    import_parser = subparsers.add_parser("import")
    import_parser.add_argument("--user", help="wordpress user", required=True)
    import_parser.add_argument("--file", help="HTML file", default="")
    import_parser.add_argument("--directory", help="HTML directory", default="")
    import_parser.add_argument("--canalblog", help="URL Canalblog", default="")
    import_parser.add_argument("--wordpress", help="URL Wordpress", required=True)

    export_parser = subparsers.add_parser("export")

    export_parser.add_argument("--url", help="canblog URL to be scraping", required=True)
    export_parser.add_argument("--directory",
                      default="backup",
                      help="backup file path")
    export_parser.add_argument("--no-css", help="No CSS", dest="css",  action="store_true")
    export_parser.add_argument("--no-js", help="No JS", dest="js",  action="store_true")
    export_parser.add_argument("--no-img", help="No img", dest="img",  action="store_true")
    export_parser.add_argument("--no-html", help="No HTML", dest="html", action="store_true")

    

    args = parser.parse_args()

    logger = logging.getLogger('import export canalblog')
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    if args.quiet is False:
        ch = logging.StreamHandler()
        if args.debug is True:
            logger.setLevel(logging.DEBUG)
            ch.setLevel(logging.DEBUG)
        else:
            logger.setLevel(logging.INFO)
            ch.setLevel(logging.INFO)
        ch.setFormatter(formatter)
        logger.addHandler(ch)


    if len(args.logfile) > 0:
        fileHandler = logging.FileHandler(args.logfile)
        if args.debug is True:
            fileHandler.setLevel(logging.DEBUG)
        else:
            fileHandler.setLevel(logging.INFO)
        fileHandler.setFormatter(formatter)
        logger.addHandler(fileHandler)

    if args.command == "import":
        password = getpass()
        if len(password) == 0:
            logger.error("No password error !!! ")
            exit(1)

        basic = HTTPBasicAuth(args.user, password)
        importWp = WPImport.WPimport(basic, args.wordpress, logger, args.parser)
        if len(args.file) > 0:
            importWp.fromFile(args.file.split(","))
            exit(0)
        if len(args.directory) > 0:
            importWp.fromDirectory(args.directory)
            exit(0)
        if len(args.canalblog) > 0:
            try:
                o = urlparse(args.canalblog)
                o = o._replace(scheme="https")
                url = o.geturl().replace(":///", "://")
            except Exception as err:
                logger.error("parsing error : {0}".format(err))
                exit(1)
            exportWp = WPExport.WPExport(url, logger, args.parser, args.directory)
            webpage = exportWp.getUrlPage()
            importWp.fromUrl(webpage)


    if args.command == "export":
        try:
            o = urlparse(args.url)
            o = o._replace(scheme="https")
            url = o.geturl().replace(":///", "://")
        except Exception as err:
            logger.error("parsing error : {0}".format(err))
            exit(1)
        exportWp = WPExport.WPExport(url, logger, args.parser, args.directory)
        if args.js is False:
            exportWp.downloadJs()
    
        if args.css is False:
            exportWp.downloadCss()
    
        if args.html is False or args.img is False:
            webpage = exportWp.getUrlPage()
        if args.html is False:
            exportWp.downloadHTML(webpage)
    
        if args.img is False:
            exportWp.downloadImg(webpage)
        exit(0)